// Modified:
//
//   - using TeX format output;
//   - included SH (Shih & Hsu's algorithm);
//   - included BM (Boyer and Myrvold's algorithm); --> removed!
//   - tested with LEDA 4.4 (LEDA 4.2 and 4.3, BL_PLANAR seems to be quadratic).
// 
//-----------------------------------------------------------------------------------
//
// LEDA-4.2
//
// planarity_time.c
//
// This file is part of the LEDA research version (LEDA-R) that can be 
// used free of charge in academic research and teaching. Any commercial
// use of this software requires a license which is distributed by the
// Algorithmic Solutions GmbH, Postfach 151101, 66041 Saarbruecken, Germany,
// http://www.algorithmic-solutions.com (contact@algorithmic-solutions.com)
//
// Copyright (c) 1991-2000  by  Max-Planck-Institut fuer Informatik
// Im Stadtwald, 66123 Saarbruecken, Germany     
// All rights reserved.
//
//-----------------------------------------------------------------------------------

#include <LEDA/graph.h>
#include <LEDA/graph_gen.h>
#include <LEDA/graph_misc.h>
#include <LEDA/plane_graph_alg.h>
#include <LEDA/string.h>
#include <LEDA/IO_interface.h>
#include <assert.h>
#include "sh-planar/sh_planar.h"

//#define _WRITE_DAT_
#define _P_   0
#define _K33_ 1
#define _K5_  2
#define _MP_  3
#define _MPE_ 4

bool
  bl_test = true,
  ht_test = true,
  bm_test = false,
  sh_test = true;
int 
  embed,
  ht_embed,
  n, m, L;
float GT, BLT, BLE, HTT, HTE, BME, BMEC, SHT, SHE, T, UT;
bool is_planar = false;

//-----------------------------------------------------------------------------------
//---- bm-planar ----
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>
#include <stdio.h>
/*********** BM removed!
#include "bm-planar/graph.h"

// ---- converts graph from gw format to bm graph format ----
void gw_to_bm_graph(graph& H, node_map<int>& number, graphP theGraph)
{
  int n = H.number_of_nodes();
  if (gp_InitGraph(theGraph, n) != OK)
    error_handler(1, "-> Program terminated (gw_to_bm_graph)!"
                     " Failed to create space for theGraph!");
  node v;
  forall_nodes(v, H)
    {
      theGraph->G[number[v]].v = number[v];
      edge e;
      forall_out_edges(e, v)
	{
	  node w = target(e);
	  if (number[v] <= number[w])
	    {
	      int result = gp_AddEdge(theGraph, number[v], 0, number[w], 0);
            if (result == NOTOK)
              error_handler(1, "-> Program terminated (gw_to_bm_graph)!"
                               " gp_AddEdge!");
            if (result == NONPLANAR)
              error_handler(1, "-> Program terminated (gw_to_bm_graph),"
                               " gp_AddEdge:NONPLANAR(EDGE_LIMIT)");
	    }
	}
    }
}

// Returns true if H is planar, false otherwise.
bool run_bm_planar(graph& H, list<edge>& K, bool check)
{
  node_map<int> number(H);
  int i = 0;
  node v;
  forall_nodes(v, H)
    number[v] = i++;

  // ---- converts graph from gw format ----
  graphP G = gp_New();
  gw_to_bm_graph(H, number, G);

  // runs bm_planar:
  int  embedFlags = EMBEDFLAGS_PLANAR;
  graphP theEmbedding=NULL;

  if ((theEmbedding = gp_New()) == NULL || gp_InitGraph(theEmbedding, G->N) != OK)
    error_handler(1, "-> Program terminated (run_bm_planar_from_gw_graph)!"
                     " Failed to create space for the embedding!");
  bool planar = false;
  int Result = gp_Embed(G, theEmbedding, embedFlags);
  switch (Result)
    {
    case OK :
      // graph is planar;
      if (check && gp_CheckEmbeddingIntegrity(theEmbedding) != OK)
	error_handler(1, "-> Program terminated (run_bm_planar_from_gw_graph)!"
		      " Failed integrity check on the embedding's vertex orientation!");
      planar = true;

      //gp_Write(theEmbedding, "justification.out", WRITE_ADJLIST);
      break;

    case NONPLANAR :
      if (check && gp_CheckKuratowskiSubgraphIntegrity(theEmbedding, G) != OK)
	error_handler(1, "-> Program terminated (run_bm_planar_from_gw_graph)!"
		      " Failed to validate Kuratowski subgraph!");
      planar = false;

      //gp_Write(theEmbedding, "justification.out", WRITE_ADJLIST);
      break;

    default :
      cout << "...Result=" << Result << "\n";
      error_handler(1, "-> Program terminated (run_bm_planar_from_gw_graph)!"
		    " The embedder failed!");
    }

  gp_Free(&G);
  gp_Free(&theEmbedding);

  return planar;
}
*/
bool run_bm_planar(graph& H, list<edge>& K, bool check)
{
  return true;
}
//-----------------------------------------------------------------------------------

void print_msg(char *msg, double val)
{
  printf("%s%.2f", msg, val);
}
void print_msg(const string& msg)
{
  cout << msg;
}
void print_times(bool test, int embed, float T, float E, int L)
{
  if (test)
    {
      print_msg(" & ", T/L);
      if (embed == 1)
	print_msg(" & ", E/L);
      else
	print_msg(" & --");
    }
  else
    {
      print_msg(" & --");
      print_msg(" & --");
    }
}
void print_header()
{
  print_msg("\n\\begin{center}\n\\begin{tabular}{lrrrrrrrrr}\n\\hline\\hline\n");
  //print_msg("Graph & Gen & \\multicolumn{2}{c}{BL\\_PLANAR}");
  print_msg("Graph & Size & \\multicolumn{2}{c}{BL\\_PLANAR}");
  print_msg(" & \\multicolumn{2}{c}{HT\\_PLANAR}");
  print_msg(" & \\multicolumn{2}{c}{BM\\_PLANAR}");
  print_msg(" & \\multicolumn{2}{c}{SH\\_PLANAR}");
  print_msg("\\\\\n\\hline\n");
  print_msg(" & & $T$ & $TJC$ & $T$ & $TJC$ & $TJ$ & $TJC$ & $T$ & $TJC$ \\\\\n\\hline\\hline\n");
}
void print_line(int n_m)
{
  //print_msg(" & ", GT/L);
  cout << " & " << n_m;
  print_times(bl_test, embed, BLT, BLE, L);
  print_times(ht_test, ht_embed, HTT, HTE, L);
  print_times(bm_test, embed, BME, BMEC, L);
  print_times(sh_test, embed, SHT, SHE, L);
  print_msg(" \\\\ \\hline\n ");
}

int read_graph(graph& G, string gw_file)
{
  int error = G.read(gw_file);
  switch (error)
    {
    case 0:
      cout << "** Graph in file \"" << gw_file 
           << "\" successfully loaded.\n";
      break;
    case 1: 
      cout << "* Error: File \"" << gw_file << "\" not found!\n";
      break;
    case 2: 
      cout << "* Error: The edge and node parameter"
           <<" types in file \"" 
           << gw_file << "\" do not match!\n";
      break;
    case 3: 
      cout << "* Error: File \"" << gw_file 
           << "\" with invalid format!\n";
      break;
    }
  return error;
}

void remove_parallel_edges_and_self_loops(graph &G)
{
  node u;
  node_map<bool> marked(G, false);
  list<edge> el;
  forall_nodes(u, G)
    {
      list<node> nl;
      marked[u] = true;
      nl.append(u);
      edge e;
      forall_out_edges(e, u)
	{
	  node v = target(e);
	  if (marked[v])
	    el.append(e);
	  marked[v] = true;
	  nl.append(v);
	}
      node w;
      forall(w, nl)
	marked[w] = false;
    }
  G.del_edges(el);
}
void make_graph(graph& G, int i, bool first_line, int l)
{
  if (i < 3)
    {
      random_planar_graph(G,n,m);

      array<node> V(6);
      for (int j = 0; j < 6; j++) V[j] = G.choose_node();

      if (i % 3 == 0 ) 
	if (first_line && l == 0) print_msg("$P$");
 
      if (i % 3 == 1)
	{
	  if (first_line && l == 0) print_msg("$P+K_{3,3}$");
	  for (int j = 0; j < 3; j++)
	    for (int k = 3; k < 6; k++) 
	      {
		G.new_edge(V[j],V[k]);
		G.new_edge(V[k],V[j]);
	      }
	}

      if ( i % 3 == 2 )
	{
	  if (first_line && l == 0) print_msg("$P+K_5$");
	  for (int j = 0; j < 5; j++)
	    for (int k = j + 1; k < 5; k++) 
	      {
		G.new_edge(V[j],V[k]);
		G.new_edge(V[k],V[j]);
	      }
	}
    }
  else
    {
      maximal_planar_graph(G,n);
  
      node v = G.choose_node();
      node w; bool _ok; edge e;

      do {
	_ok = true;
	w = G.choose_node();
	forall_adj_edges(e,v)
	  if ( G.target(e) == w ) _ok = false; 
      } while ( _ok == false );
      if ( i == 3 )
	{
	  if (first_line && l == 0)
	    print_msg("$P^*$");
	}
      else
	{
	  G.new_edge(v,w);
	  G.new_edge(w,v);
	  if (first_line && l == 0)
	    print_msg("$P^*+e$");
	}
    }
  remove_parallel_edges_and_self_loops(G);
  G.make_bidirected();
}

void run_impls(graph& G)
{
  // ---- bl_planar ----
  if (bl_test)
    {
      list<edge> K;
      graph H;
      CopyGraph(H, G); // makes a copy H of G;
      list<edge> E = H.all_edges();
      E.permute();
      H.sort_edges(E);
      H.make_map();
      used_time(T);

      BL_PLANAR(H);
      UT = used_time(T); BLT +=UT;

      if (embed == 1)
	{
	  is_planar = BL_PLANAR(H, K, true);

	  if ( is_planar )
	    assert( Genus(H) == 0 );
	  else
	    assert( CHECK_KURATOWSKI(H, K) );

	  UT = used_time(T); BLE += UT;
	}
    }

  // ---- ht_planar ----
  if (ht_test)
    {
      list<edge> K;
      graph H;
      CopyGraph(H, G); // makes a copy H of G;
      list<edge> E = H.all_edges();
      E.permute();
      H.sort_edges(E);
      H.make_map();
      used_time(T);

      is_planar = HT_PLANAR(H);
      if (!is_planar)
	ht_embed = 0;
      else
	ht_embed = embed;
      UT = used_time(T); HTT += UT;

      if (ht_embed == 1)
	{
	  is_planar = HT_PLANAR(H, K, true);

	  if ( is_planar )
	    assert( Genus(H) == 0 );
	  //else I.write_demo("no efficient Kuratowski finder in HT_PLANAR ");

	  UT = used_time(T); HTE += UT;
	}
    }

  // ---- bm_planar ----
  if (bm_test)
    {
      list<edge> K;
      graph H;
      CopyGraph(H, G); // makes a copy H of G;
      list<edge> E = H.all_edges();
      E.permute();
      H.sort_edges(E);
      H.make_map();
      used_time(T);

      // T+J, but without checking the justification;
      is_planar = run_bm_planar(H, K, false);
      UT = used_time(T); BME += UT;

      if (embed == 1)
	{
          // T+J and checking the justification;
	  is_planar = run_bm_planar(H, K, true);

          /* nao consegui fazer a traducao da justificativa **********
	  if ( is_planar )
	    assert( Genus(H) == 0 );
	  else
	    assert( CHECK_KURATOWSKI(H, K) );
	  */

	  UT = used_time(T); BMEC += UT;
	}
    }

  // ---- sh_planar ----
  if (sh_test)
    {
      list<edge> K;
      graph H;
      CopyGraph(H, G); // makes a copy H of G;
      list<edge> E = H.all_edges();
      E.permute();
      H.sort_edges(E);
      H.make_map();
      used_time(T);

      SH_PLANAR(H);
      UT = used_time(T); SHT +=UT;

      if (embed == 1)
	{
	  is_planar = SH_PLANAR(H, K, true);

	  if ( is_planar )
	    assert( Genus(H) == 0 );
	  else
	    assert( CHECK_KURATOWSKI(H, K) );

	  UT = used_time(T); SHE += UT;
	}
    }
}

void read_file_name(const string& mes, string& file_name)
{
  cout << "\n\n" << mes;
  file_name.read_line();
  file_name.read_line();
}


int main() { {  // I want two blocks so that statistics is outside

  IO_interface_demo R("Input");

  embed = R.read_int("Please, type 1 for embedding: ", 0),
  ht_embed = embed;

  graph G;

  leda_string file_name("");
  read_file_name("Please, specify a graph (gw file) or just press ENTER: ", file_name);
  if (read_graph(G, file_name))
  //if (G.read(file_name) != 0)
    {
#ifdef _WRITE_DAT_
  FILE *pdat = NULL, *k33dat = NULL,
       *k5dat = NULL, *mpdat = NULL, *mpedat = NULL;
  if ((pdat = fopen("./gnuplot/p.dat", "wt")) == NULL ||
      (k33dat = fopen("./gnuplot/k33.dat", "wt")) == NULL ||
      (k5dat = fopen("./gnuplot/k5.dat", "wt")) == NULL ||
      (mpdat = fopen("./gnuplot/mp.dat", "wt")) == NULL ||
      (mpedat = fopen("./gnuplot/mpe.dat", "wt")) == NULL)
    error_handler(1, "-> Program terminated! (Could not create dat files!)");
#endif

      cout << "Random graphs selected...\n";

      int N = R.read_int("number of nodes = ",100);
      bool first_line = true;
      int total = R.read_int("number of lines = ",10);
      L = R.read_int("number of graphs per line = ",10);

      print_header();

      for (int i = 0; i < 5; i++)
	{ 
	  for (n = N; n <= pow(2,total-1)*N; n  *= 2)
	    {
	      first_line = (n == N);
	      GT = BLT = BLE = HTT = HTE = BME = BMEC = SHT = SHE = 0;
	      m = 2*n;
	      for (int l = 0; l < L; l++)
		{
		  T = used_time();
		  make_graph(G, i, first_line, l);
		  UT = used_time(T); GT += UT;

		  run_impls(G);
		}
	      print_line(n);
#ifdef _WRITE_DAT_
	      FILE *f = NULL;
	      switch(i)
		{
		case _P_:
		  f = pdat;
		  break;
		case _K33_:
		  f = k33dat;
		  break;
		case _K5_:
		  f = k5dat;
		  break;
		case _MP_:
		  f = mpdat;
		  break;
		case _MPE_:
		  f = mpedat;
		  break;
		}
	      if (f != NULL)
	        fprintf(f, "%d %.2f %.2f %.2f %.2f %.2f %.2f %.2f %.2f\n",
		        n, BLT/L, BLE/L, HTT/L, HTE/L, BME/L, BMEC/L, SHT/L, SHE/L);
#endif
	    }
	}
#ifdef _WRITE_DAT_
  fclose(pdat);
  fclose(k33dat);
  fclose(k5dat);
  fclose(mpdat);
  fclose(mpedat);
#endif
    }
  else
    {
      GT = BLT = BLE = HTT = HTE = BME = SHT = SHE = 0;
      L = 1;
      T = used_time();
      G.make_bidirected();
      n = G.number_of_nodes();
      m = G.number_of_edges()/2;
      print_header();
      UT = used_time(T); GT += UT;
      run_impls(G);
      print_line(n+m);
    }
  print_msg("\\hline\n\\end{tabular}\n\\end{center}\n");

#ifdef _WRITE_DAT_
  cout << "Outputs in directory './gnuplot': p.dat, k33.dat, k5.dat, mp.dat, mpe.dat\n";
#endif
}

//print_statistics();

 return 0;
}
