//-----------------------------------------------------------------------------------
// Modules: sh_planar.h, sh_impl.h, sh_test.c, sh_aux.c, sh_embed.c, sh_obstr.c, 
//          dfspos.c, dfspos.h.
//
//Credits:
//--------
//  
//  Alexandre Noma                <noma@ime.usp.br> -- supported by Fapesp.
//  Cristina Gomes Fernandes      <cris@ime.usp.br>
//  John M. Boyer                 <jboyer@acm.org>
//  Jose Coelho de Pina Junior    <coelho@ime.usp.br>
//
//  Institute of Mathematics and Statistics - University of Sao Paulo - Brazil
//-----------------------------------------------------------------------------------

#ifndef _SH_PLANAR_H
#define _SH_PLANAR_H

#include "sh_impl.h"

#if defined(LEDA_NAMESPACE)
using namespace leda;
#endif

//-----------------------------------------------------------------------------------
// Tests if a graph G is planar.
// If G is planar and embed == true and G is a map, then G is turned into a plane map.
// If G is not planar and embed == true and G is a map, then obstr_edges contains the
// edges of a subdivision of K_{3,3} or K_5 in G (the construction of obstr_edges
// depends on the partial embedding of G).
// Returns true if G is planar, 
//         false otherwise.
//
// Precondition:
// - G must be a bidirected graph;
// - to embed G, G must be loopfree and have no parallel edges.
//-----------------------------------------------------------------------------------
bool SH_PLANAR(
	       graph_type&,
               list<edge>& obstr_edges,
               bool embed);
bool SH_PLANAR(graph_type&);

#endif
